uniform vec2 resolution;
uniform float time;
uniform vec4 color;

const float PI = 3.14159265;
const float cSpeed = 0.05;

vec2 hash2(vec2 p)
{
	// texture based white noise
	//	return texture( iChannel0, (p+0.5)/256.0, -100.0 ).xy;

	// procedural white noise
	return fract(sin(vec2(dot(p, vec2(127.1, 311.7)), dot(p, vec2(269.5, 183.3))))*43758.5453);
}

vec3 voronoi(in vec2 x)
{
	vec2 n = floor(x);
	vec2 f = fract(x);

	//----------------------------------
	// first pass: regular voronoi
	//----------------------------------
	vec2 mg, mr;

	float md = 8.0;
	for (int j = -1; j <= 1; j++)
		for (int i = -1; i <= 1; i++)
		{
			vec2 g = vec2(float(i), float(j));
			vec2 o = hash2(n + g);
			o = 0.5 + 0.5*sin(time + 6.2831*o);
			vec2 r = g + o - f;
			float d = dot(r, r);

			if (d<md)
			{
				md = d;
				mr = r;
				mg = g;
			}
		}

	//----------------------------------
	// second pass: distance to borders
	//----------------------------------

	md = 8.0;
	for (int j = -2; j <= 2; j++)
		for (int i = -2; i <= 2; i++)
		{
			vec2 g = mg + vec2(float(i), float(j));
			vec2 o = hash2(n + g);
			o = 0.5 + 0.5*sin(time + 6.2831*o);
			vec2 r = g + o - f;

			if (dot(mr - r, mr - r)>0.00001)
				md = min(md, dot(0.5*(mr + r), normalize(r - mr)));
		}


	return vec3(md, mr);
}


float sphere(float t, float k)
{
	float d = 1.0 + t*t - t*t*k*k;
	if (d <= 0.0)
		return -1.0;
	float x = (k - sqrt(d)) / (1.0 + t*t);
	return asin(x*t);
}


/* NOISE */


// NOISE APPEARANCE TESTS
// ORIGINAL : https://www.shadertoy.com/view/4sc3z2

#define MOD3 vec3(.1031,.11369,.13787)
vec3 hash33(vec3 p3)
{
	p3 = fract(p3 * MOD3);
	p3 += dot(p3, p3.yxz + 19.19);
	return -1.0 + 2.0 * fract(vec3((p3.x + p3.y)*p3.z, (p3.x + p3.z)*p3.y, (p3.y + p3.z)*p3.x));
}

float simplexNoise(vec3 p)
{
	const float K1 = 0.333333333;
	const float K2 = 0.166666667;

	vec3 i = floor(p + (p.x + p.y + p.z) * K1);
	vec3 d0 = p - (i - (i.x + i.y + i.z) * K2);

	// thx nikita: https://www.shadertoy.com/view/XsX3zB
	vec3 e = step(vec3(0.0), d0 - d0.yzx);
	vec3 i1 = e * (1.0 - e.zxy);
	vec3 i2 = 1.0 - e.zxy * (1.0 - e);

	vec3 d1 = d0 - (i1 - 1.0 * K2);
	vec3 d2 = d0 - (i2 - 2.0 * K2);
	vec3 d3 = d0 - (1.0 - 3.0 * K2);

	vec4 h = max(0.6 - vec4(dot(d0, d0), dot(d1, d1), dot(d2, d2), dot(d3, d3)), 0.0);
	vec4 n = h * h * h * h * vec4(dot(d0, hash33(i)), dot(d1, hash33(i + i1)), dot(d2, hash33(i + i2)), dot(d3, hash33(i + 1.0)));

	return dot(vec4(31.316), n);
}

float noise(vec3 p)
{
	return simplexNoise(p);
}

float noiseSum(vec3 p)
{
	float f = 0.0;
	p = p * 4.0;
	f += 1.0000 * noise(p); p = 2.0 * p;
	f += 0.5000 * noise(p); p = 2.0 * p;
	f += 0.2500 * noise(p); p = 2.0 * p;
	f += 0.1250 * noise(p); p = 2.0 * p;
	f += 0.0625 * noise(p); p = 2.0 * p;

	return f;
}

float noiseSumAbs(vec3 p)
{
	float f = 0.0;
	p = p * 3.0;
	f += 1.0000 * abs(noise(p)); p = 2.0 * p;
	f += 0.5000 * abs(noise(p)); p = 2.0 * p;
	f += 0.2500 * abs(noise(p)); p = 2.0 * p;
	f += 0.1250 * abs(noise(p)); p = 2.0 * p;
	f += 0.0625 * abs(noise(p)); p = 2.0 * p;

	return f;
}


void main()
{
	vec2 uv = gl_FragCoord.xy - 0.5*resolution.xy;
	float v = resolution.x;
	uv /= v;
	vec2 originalUV = uv;

	uv *= 3.0;
	float len = length(uv) * 0.8;
	float k = 1.0;
	float len2;

	len2 = sphere(len*k, sqrt(2.0)) / sphere(1.0*k, sqrt(2.0));
	uv = uv * len2 * 0.5 / len;
	uv = uv + 0.5;

	vec2 pos = uv;
	float t = time / 1.0;
	float scale1 = 40.0;
	float scale2 = 20.0;
	float val = 0.0;

	val += sin((pos.x*scale1 + t));
	val += sin((pos.y*scale1 + t) / 2.0);
	val += sin((pos.x*scale2 + pos.y*scale2 + sin(t)) / 2.0);
	val += sin((pos.x*scale2 - pos.y*scale2 + t) / 2.0);
	val /= 2.0;


	vec3 c = voronoi(64.0*pos);

	// isolines
	//val += 2.0*sin(t)*c.x*(0.5 + 0.5*sin(64.0*c.x));

	float glow = 0.05 / (0.015 + distance(len, 1.0));

	val = (cos(PI*val) + 1.0) * 0.5;

	vec4 result = step(len, 1.0) * 0.5 * color * val + glow * color;

	// Display shield or not
	float dst = distance(originalUV, vec2(0.0, 0.0));
	float currentSin = sin(time * 1.05) * 1.5;

	// Adding transparency
	if (dst > 0.42)
	{
		result = glow * color;
		result.a = 0.25;
	}
	else if (val < 0.3)
		result.a = 0.0;

	// Appear and disappear effect
	float f = noiseSumAbs(vec3(uv * 1.0, time * cSpeed) * 1.0);
	if (f > currentSin)
		result.a = 0.0;

	gl_FragColor = result;
}